#ifndef __File_DiamondCutting_H
#define __File_DiamondCutting_H

/*  File DiamCut.h

    Interface of diamond cut

    Copyright (c) 2000 by OctoNus Ltd.
    All rights reserved.
*/

#include "windows.h"

#pragma pack (push, 4)

#include "Vector.h"

typedef unsigned short PHINDEX;
#define MAXPHINDEX 0xFFFF

// Structure of cut data description
struct ClassDiamondCutting
{
        int   dataSize;         // Size of data for the object
        CLSID uid;              // Unique interface identifier
        int   version;          // Version of cut interface
        const PHINDEX * sides;  // Distribution of points along facets
        const char * name;      // Cut name
        const char * shortname; // Name of cut instances
};

// Instance of the cut
struct DiamondCutting
{
        ClassDiamondCutting * classDiamond;     // cut class
        void * data;    // own cut data
        int dataSize;   // size of own cut data
        POINT3 * vertex;// array of diamond vertexes
};

#define PF_IOFUNC   0x01
#define PF_FONT     0x02
#define PF_DIGIT    0x04
#define PF_LEFT     0x08
#define PF_RIGHT    0x10

// Format of parameter presentation in control panel
struct ParameterFormat
{
        int state;      // description of fields in this structure (combination of PF_* flags)
        int ioFunc;     // (PF_IOFUNC) DC_PFPercent, DC_PFAngle, DC_PFMetric, DC_PFNumber
        int font;       // (PF_FONT) DC_PFFontSystem, DC_PFFontSymbol
        int digit;      // (PF_DIGIT) number of digits after decimal point
        double left;    // (PF_LEFT) lower boundary in the panel
        double right;   // (PF_RIGHT) upper boundary in the panel
};

enum
{
        DC_PFPercent = 1,// percents
        DC_PFAngle,     // angle
        DC_PFMetric,    // millimeters
        DC_PFNumber,    // number (usually a ratio)
        DC_PFFontSystem,// standard font
        DC_PFFontSymbol,// Symbol font
};

// Definition of standard formats
#define PFDefault   ((ParameterFormat*)(-1))
#define PFPercent   ((ParameterFormat*)(-2))
#define PFAngle     ((ParameterFormat*)(-3))
#define PFMetric    ((ParameterFormat*)(-4))
#define PFNumber    ((ParameterFormat*)(-5))
#define PFAngleS    ((ParameterFormat*)(-6))
#define PFPercentSym ((ParameterFormat*)(-7))
#define PFPercent0  ((ParameterFormat*)(-8))

// Presentation of parameter grade in the control panel
// As different parameter appraisers evaluate contain different parameters,
// e.g. one can have pavilion height and another pavion angle,
// an order of evaluation can be specified for each parameter.

typedef int * ColorList;

#define CLDefault   ((ColorList)(-1))

// The way of presenting the parameter in the panel
struct ParameterInfo
{
        int parameter;          // Parameter name
        const char * name;      // Displayed parameter name in the the diamond panel
        ParameterFormat * format; // Input/output format of the parameter
        ColorList colorList;    // Order of parameter evaluation
};

// Parameter list for presentation in the diamond panel
struct ParameterList
{
        int bufSize;    // size of parameter list buffer
        int count;      // quantity of filled parameters
        int listType;   // identifier of parameter list type (Standard, Full...)
        int * pars;     // buffer for parameter list
};

// Lists of different parameter presentations
enum
{
        DC_StandardParameterList,
        DC_FullParameterList,
        DC_ReportParameterList,
};

// Types of exported functions
typedef int (*fnCuttingList) (int * cuts, int n);
typedef int (*fnCuttingInfo) (void * data, int cut, int flavor);
typedef int (*fnInitDiamond) (DiamondCutting * diam, int cut);
typedef int (*fnInitVertex) (DiamondCutting * diam, int cut);
typedef int (*fnIsCorrect) (DiamondCutting * diam, int cut);
typedef int (*fnGetParameter) (double * value, int * param, int nparams,
        DiamondCutting * diam, int cut);
typedef int (*fnSetParameter) (double * value, int * param, int nparams,
        DiamondCutting * diam, int cut);

// CuttingInfo function query
enum
{
        DC_ClassDiamondCutting = 0x100, // cut class
        DC_OptimizationInfo,
        DC_CheckConvex,
        DC_ParameterInfo,    // Information on parameter
        DC_ParameterList,    // Lists of parameters presentation

        DC_OutputStringInterface = 0x7FFF, // Temporary Output string func

};

// Output string function type
extern "C"
{
typedef int (*PrintfFuncType)(const char *, ...);
}

#define DC_PARAMETERS_START_V1 100
#define DC_PARAMETERS_START_V2 0x1000

#if ! defined (DC_PARAMETERS_VERSION) || DC_PARAMETERS_VERSION == 1
#define DC_PARAMETERS_START DC_PARAMETERS_START_V1
#else
#define DC_PARAMETERS_START DC_PARAMETERS_START_V2
#endif

// List of diamond parameters

enum
{
        // Main parameters
        DC_Diameter       =  1, // diameter of the diamond, mm
        DC_AlternateDiameter = 32, // alternate diameter of the diamond, mm
        DC_GirdleRatio    = 10, // ration of lenght to width
        DC_GirdleThickness= 37, // girdle thickness, % of D
        DC_CrownAngle     = 43, // crown angle;
                                // when it is set, the crown height
                                // (DC_CrownHeight) is fixed
        DC_CrownAngle_Table = 3, // crown angle;
                                // when it is set, the table diameter
                                // (DC_TableDiameter) is fixed
        DC_CrownHeight    = 38, // crown height, % of D
        DC_TableDiameter  =  7, // table diameter, % of D
        DC_PavilionAngle  =  4, // pavilion angle
        DC_PavilionDepth  = 48, // pavilion depth, % of D
        DC_LowerFacets    = 18, // level of lower pavilion facets, % of pavilion depth
        DC_TotalHeight    = 19, // total height of the stone, % of D
        DC_Culet          = 40, // culet width, % of D
        DC_CuletX         = 56, // shift of culet to the direction of main diameter, % of D
        DC_CuletY         = 57, // shift of culet to the diretion of stretching, % of D

        // Additional parameters
        DC_UserParameter  = DC_PARAMETERS_START,    // other parameters, defined by the user
        DC_UserParameterPercent =   // other parameters that have default
            DC_UserParameter + 10,  // presentation in % of D
        DC_UserParameterAngle =     // other parameters that have default
            DC_UserParameter + 20,  // presentation as an angle
        DC_UserParameterNumber =    // other parameters that have default
            DC_UserParameter + 30,  // presentation as a number (ratio)
        DC_UserParameterMetric =    // other parameters that have default
            DC_UserParameter + 40,  // presentation in mm
        DC_UserParameterOther =
            DC_UserParameter + 50,
        DC_EndOfUserParameter =     // the last allowable parameter
            DC_UserParameter + 1000,

        // Redefinition of the list for checking parameter quality
        DC_QualityDefault =         //
            DC_UserParameter + 1001,//
        DC_QualityLast,             // end of redefinition list
};

#pragma pack (pop)

#endif //__File_DiamondCutting_H
